/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function RegExpPage() // extend WizardPage
{
    var depends = "images script";
    WizardPage.call(this, depends);
    
    this.pattern.parent = this;
    this.template.parent = this;
}

RegExpPage.prototype =
{
    get source()      { return $("rgroup_source"); },
    get sourceValue() { return parseInt(this.source.value); },
    
    get sourceName() { return ((this.sourceValue == TYPE_REGEXP_THUMBNAIL) ? "thumbnail" : "link"); },
    
    sourceNames: {image: "thumbnail", anchor: "link"},
    
    pattern:
    {
        parent: null,
        get thumbnail() { return $("tbox_pattern_thumbnail"); },
        get link()      { return $("tbox_pattern_link"); },
        get selected()  { return this[this.parent.sourceName]; },
    },
    
    template:
    {
        parent: null,
        get thumbnail() { return $("tbox_template_thumbnail"); },
        get link()      { return $("tbox_template_link"); },
        get selected()  { return this[this.parent.sourceName]; },
    },
    
    onPageShow: function()
    {
        // populate listboxes
        if (this.dirty.images)
        {
            for each (var name in ["image", "anchor"])
            {
                var sourceName = this.sourceNames[name];
                
                var rlbox = $("rlbox_regexp_" + sourceName);
                while (rlbox.itemCount > 0)
                    rlbox.removeItemAt(0);
                for each (var thumbnail in model.images)
                    rlbox.appendItem(thumbnail[name].link);
                
                var rlbox = $("rlbox_regexp_" + sourceName + "_images");
                while (rlbox.itemCount > 0)
                    rlbox.removeItemAt(0);
                for each (var thumbnail in model.images)
                    rlbox.appendItem(thumbnail.target.image.link, thumbnail[name].link);
            }
        }
        
        // pattern & template
        if (this.dirty.images || this.dirty.script)
        {
            for each (var name in ["image", "anchor"])
            {
                var sourceName = this.sourceNames[name];
                var source = window["TYPE_REGEXP_" + sourceName.toUpperCase()];
                
                var script = model.script;
                if (! (script && (script.type == TYPE_REGEXP) && (script.subType == source)))
                    script = null;
                
                var pattern = (script ? script.pattern : scriptWizard.getPattern(name, model.images));
                var template = (script ? script.template : scriptWizard.getTemplate(name, model.images, pattern));
                
                this.pattern[sourceName].defaultValue = pattern;
                this.pattern[sourceName].reset();
                
                this.template[sourceName].defaultValue = template;
                this.template[sourceName].reset();
            }
        }
        
        if (this.dirty.page)
        {
            this._updateButtons();
            this._updatePattern();
            this._updateTemplate();
        }
    },
    
    onSourceSelect: function()
    {
        $("deck_pattern").selectedIndex = this.sourceValue - 1;
        $("deck_template").selectedIndex = this.sourceValue - 1;
        $("deck_sources").selectedIndex = this.sourceValue - 1;
        $("lbl_pattern").control = "tbox_pattern_" + this.sourceName;
        $("lbl_template").control = "tbox_template_" + this.sourceName;
        
        this._updateButtons();
        this._updatePattern();
        this._updateTemplate();
        wizard.updateWizardButtons();
    },
    
    onPatternInput: function()
    {
        if (this.pattern.updateTimer)
            this.pattern.updateTimer.cancel();
        
        this.pattern.updateTimer = callLater(INPUT_DELAY, this, this._updatePattern);
        
        this._updateButtons();
        wizard.updateWizardButtons();
    },
    
    onTemplateInput: function()
    {
        if (this.template.updateTimer)
            this.template.updateTimer.cancel();
        
        this.template.updateTimer = callLater(INPUT_DELAY, this, this._updateTemplate);
        
        this._updateButtons();
        wizard.updateWizardButtons();
    },
    
    resetPattern: function()
    {
        this.pattern.selected.reset();
        this._updateButtons();
        this._updatePattern();
        wizard.updateWizardButtons();
    },
    
    resetTemplate: function()
    {
        this.template.selected.reset();
        this._updateButtons();
        this._updateTemplate();
        wizard.updateWizardButtons();
    },
    
    canAdvance: function()
    {
        if (this.pattern.updateTimer || this.template.updateTimer)
            return false;
        
        if (! $("rlbox_regexp_" + this.sourceName).match)
            return false;
        
        if (! $("rlbox_regexp_" + this.sourceName + "_images").match)
            return false;
        
        return true;
    },
    
    _updateButtons: function(sourceName)
    {
        $("btn_pattern_reset").disabled = (this.pattern.selected.value == this.pattern.selected.defaultValue);
        $("btn_template_reset").disabled = (this.template.selected.value == this.template.selected.defaultValue);
    },
    
    _updatePattern: function()
    {
        if (this.pattern.updateTimer)
        {
            this.pattern.updateTimer.cancel();
            delete this.pattern.updateTimer;
        }
        
        if (page != pages.regexp)
            return;
        
        var pattern = "^" + this.pattern.selected.value + "$";
        $("rlbox_regexp_" + this.sourceName).pattern = pattern;
        $("rlbox_regexp_" + this.sourceName + "_images").pattern = pattern;
        
        wizard.updateWizardButtons();
    },
    
    _updateTemplate: function()
    {
        if (this.template.updateTimer)
        {
            this.template.updateTimer.cancel();
            delete this.template.updateTimer;
        }
        
        if (page != pages.regexp)
            return;
        
        var template = this.template.selected.value;
        $("rlbox_regexp_" + this.sourceName + "_images").template = template;
        
        wizard.updateWizardButtons();
    },
    
    onPageAdvanced: function()
    {
        model.type = TYPE_REGEXP;
        model.source = this.sourceValue;
        model.pattern = this.pattern.selected.value;
        model.template = this.template.selected.value;
        return wizard.advance("script");
    }
}

extend(RegExpPage, WizardPage);

pages.regexp = new RegExpPage();
